/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// ***************************************************************************
// Atom Class
// 
//
// Class Description:  
// ------------------
// Purpose:  This class is the main interface for the whole system.
// It instantiates and lays out the graphical user interface (GUI) components 
// found on the first interface the user will see.  It implements its own event 
// handling and allows execution of the whole system.  If this interface is 
// destroyed all subsequent interfaces called by it are destroyed.
//
// System Relationship:  This class creates the main interface for the whole 
// system.  Depending on the user event, Atom will do one of the following:
// construct the ToolInterface.class, construct the SclInterface.class, or
// execute system commands.  The system command execution is dependent on 
// audit tools assigned to a Security Classification Level (SCL).  This class
// performs file input/output.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.0.
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  None.
//
// Post Conditions: None.
//
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class must layout the interface components on the screen. It must
// respond to user generated events.  The following is a list of requirements
// implemented by this class:
//
//    *  Get the present working directory
//    *  Instantiate java.awt components used by the GUI
//    *  Layout the java.awt components
//    *  Provide event handling for the java.awt components
//
//
// Constructor
// -----------
//      public Atom(String title)
//
//
// Methods
// -------
//      public void createRegisteredAuditToolsList ()      
//      public void createSclStatusList ()
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height,
//                      int fill, int anchor, double weight_x, double weight_y,   
//                      int top, int left, int bottom, int right)
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height) 
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height,
//                      int top, int left, int bottom, int right)
//
// Event Handling 
// --------------
//      public boolean handleEvent(Event evt)
//      public boolean action(Event event, Object what)
//
//
// External Classes Used
// ---------------------
//      FileIO
//      Box
//
// *****************************************************************************

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.event.*;

public class Atom extends Constrain implements ActionListener, ItemListener {

  // *******************************************
  // Interface Component Declarations
  // *******************************************
  MenuBar  menubar;                // Menu Bar
  Menu     file, options, help;    // Menu Panes
  MenuItem quitItem, setDefaultItem;    // Options
  
  Panel   toolStatusPanel;         // Sub-container for Audit Tool Status, West
  Panel   sclTypesPanel;           // Sub-container for SCL Types, East
  Panel   systemStatusPanel;       // Sub-container for System Status, South
  Panel   mainOptionsPanel;        // Sub-container for Main Options, Center

  Box     statusBox;               // Container for audit tool status components
  Box     sclBox;                  // Container for SCL components
  Box     mainOptionsBox;          // Container for main options

  static  List    registeredToolsList;   // List for Audit Tool Status
  static  List    sclTypesList;          // List for SCL Types

  Button  miStartButton;           // Main interface start button
  Button  miStopButton;            // Main interface stop button
  Button  miToolButton;            // Main interface tool button
  Button  miSclButton;             // Main interface SCL button

  Label   systemStatusLabel;                     // Label for the system status
  static TextArea systemStatusTextArea;          // Text area for the system status
  GridBagLayout  gridbg  = new GridBagLayout();  // Layout Manager for components
  

  // *******************************************
  // Data Declarations
  // *******************************************

  public static String sclTypesFile;      // Location of scl_types.dat file
  public static String sclTempFile;       // Temp file 
  public static String auditToolsFile;    // Location of audit_tools.dat file
  public static String auditTempFile;     // Temp file
 
  public static String sclLevelDir;       // Location of scl_Name file(s)
  public static String sclLevelTemp;      // Temp file
  public static String scriptsDir;        // Location of scripts 
  public static String startScript;       // Script to start audit tools
  public static String stopScript;        // Script to stop audit tools

  public static String stdoutLog;         // File to capture stdout from scripts
  public static String stderrLog;         // File to capture stderr from scripts

  public static String directory;                       // Present working directory
  public static String toolClass = "System Security";   // Audit Tool Class Type
  
  // Addition - Dean - 4/5/98.
  // Internal lists to speed up performance.
  public static  String[] registeredTools = new String[0];
  public static String[] registeredSCLs  = new String[0];

  private String[] listSelection   = new String[0];     // List selection elements
 

  // *******************************************
  // Object Instantiations 
  // *******************************************
  FileIO      atomIO   = new FileIO();



  // *****************************************************************************
  // public Atom(String title)
  //
  // Constructor Description:  The constructor for the Atom class.  Initializes new
  // variables and components, and lays out the components on the main interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the main interface.  It uses the GridBagLayout Manager.  In addition
  // it defines the variables from the Data Declarations.  The following is a listing
  // of the interface layout and its corresponding components:
  // 
  //   Position  Description             Components
  //   --------  -----------             ----------
  //    * Frame  Main Interface:         GridBagLayout Manager
  //    * Menu   MenuBar:                menubar
  //    * West   Registered Audit Tools: toolStatusPanel, registeredToolsList
  //    * Center Main Screen Options:    mainOptionsPanel, miStartButton, miStopButton,
  //                                     miToolButton, miSclButton
  //    * East   SCL Types:              sclTypesPanel, sclTypesList
  //    * South  System Status:          systemStatusPanel, systemStatusTextArea
  //
  // The constructor will only be accessed once, at startup.
  // 
  // Pre-Conditions:  None.
  //
  // Post-Conditions: None.
  //
  // *****************************************************************************

  public Atom () {
    int z =0;
    setTitle("Audit Tool Manager");    
    this.setLayout(gridbg);

    addWindowListener(new WinAdapter());

    registeredToolsList = new List(12, true);
    registeredToolsList.addItemListener(this);

    sclTypesList = new List(12, true);
    sclTypesList.addItemListener(this);

   // *** Get present working directory of Atom. ***
   String pwd = null;
   pwd = System.getProperty("user.dir");
  
   // *** Initialize Data Declarations
   directory       = pwd;
   sclTypesFile    = pwd+"/config/scl_types.dat";
   sclTempFile     = pwd+"/config/stemp.tmp";
   auditToolsFile  = pwd+"/config/audit_tools.dat";
   auditTempFile   = pwd+"/config/atemp.tmp";
   sclLevelDir     = pwd+"/levels/";
   sclLevelTemp    = pwd+"/ltemp.tmp";

   // ***  Replace start, stop, stdout, stderr script when 
   //      Java system execution is operable 
   scriptsDir      = pwd+"/scripts/";
   startScript     = pwd+"/scripts/startTools";
   stopScript      = pwd+"/scripts/stopTools";
   stdoutLog       = pwd+"/stdout.log";
   stderrLog       = pwd+"/stderr.log";
  
   // *** Initialize Buttons ****
   miStartButton         = new Button("START");
   miStopButton          = new Button("STOP");
   miToolButton          = new Button("TOOL");
   miSclButton           = new Button("SCL");


   // *** BUTTON LISTENERS ***
   miStartButton.addActionListener(this);
   miStopButton.addActionListener(this);
   miToolButton.addActionListener(this);
   miSclButton.addActionListener(this);


   // *** Set Font and Color of Buttons ****
   miStartButton.setFont(new Font("Dialog", Font.BOLD, 14));   
   miStopButton.setFont(new Font("Dialog", Font.BOLD, 14));
   miToolButton.setFont(new Font("Dialog", Font.BOLD, 14));
   miSclButton.setFont(new Font("Dialog", Font.BOLD, 14));

    
    // *** Create the Menu Bar **** 
     menubar = new MenuBar();
     this.setMenuBar(menubar);

        file = new Menu("File");
	file.add(quitItem = new MenuItem("Quit"));		  
	menubar.add(file);

	options = new Menu("Options");
	options.add(setDefaultItem = new MenuItem("Set Defaults"));
	menubar.add(options);

	help = new Menu("Help");
	help.add(new MenuItem("About"));
	menubar.add(help);
	menubar.setHelpMenu(help);

     
    // *** MENUITEM LISTENERS ***
    quitItem.addActionListener(this);
    setDefaultItem.addActionListener(this);
    

    // *** Registered Audit Tools ***
    createRegisteredAuditTools();

    //    for (z=0; z<registeredTools.length; z++) {
    //      System.out.println(registeredTools[z]);
    //    }

    toolStatusPanel               = new Panel();
 
    toolStatusPanel.setLayout(gridbg);
    constrain(toolStatusPanel, registeredToolsList, 0,0,1,1);
    statusBox = new Box(toolStatusPanel, "Registered Tools");
    statusBox.etchedOut();
   

    // *** Main Screen Options ****
    mainOptionsPanel = new Panel();

    mainOptionsPanel.setLayout(new GridLayout(4,0,0,15));
      mainOptionsPanel.add(miStartButton);
      mainOptionsPanel.add(miToolButton);
      mainOptionsPanel.add(miSclButton);
      mainOptionsPanel.add(miStopButton);
    mainOptionsBox = new Box(mainOptionsPanel, "Options");
    mainOptionsBox.etchedIn();


    // *** SCL Types ****
    createSclStatusList();
    sclTypesPanel                 = new Panel();
    
    sclTypesPanel.setLayout(gridbg);
    constrain(sclTypesPanel, sclTypesList, 0,0,1,1);
    sclBox = new Box(sclTypesPanel, "Security Levels");
    sclBox.etchedOut();

    
    // *** System Status ****
    Panel  systemStatusPanel = new Panel();
    systemStatusTextArea = new TextArea(6,50);

    systemStatusPanel.setLayout(gridbg);
    constrain(systemStatusPanel, new Label("System Status"), 0,0,1,1);
    constrain(systemStatusPanel, systemStatusTextArea, 0,1,1,1);
 

    // *** Layout components on interface ***
    constrain(this, statusBox, 0,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 15, 5, 5, 10);
    constrain(this, mainOptionsBox, 1,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 15, 5, 5, 5);
    constrain(this, sclBox, 2,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 15, 10, 5, 10);
    constrain(this, systemStatusPanel, 0,1,3,1, GridBagConstraints.HORIZONTAL,
	      GridBagConstraints.CENTER, 1.0, 1.0, 15, 5, 5, 5);

  }  // Atom Constructor



  // ***************************************************************************** 
  // public boolean action(Event event, Object what)
  //
  // Method Description:  This method handles all of the events (ACTION_EVENT) 
  // generated by the components in the main interface.
  //
  // Method Specification:  If an ACTION_EVENT occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ACTION_EVENT:
  //
  //        Label        Component/Event    Reaction
  //        -----        ---------------    --------
  //      * SCL Start    miStartButton      Start audit tools for the selected 
  //                                        Scl Type     
  //      * SCL Stop     miStopButton       Stop audit tools for the selected
  //                                        Scl Type
  //      * Tool         miToolButton       Launch the ToolInterface Window
  //      * SCL          miSclButton        Launch the SclInterface Window
  //      * Menu         quitItem           Exit 
  //                     setDefaultItem     Launch DefaultWindow
  //
  // This method will be accessed many times.
  //
  // Pre-Condition:  Component objects have been instantiated.
  //
  // Post-Condition: None.
  //
  // *****************************************************************************


  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    // *** Start SCL's ***
    if (source == miStartButton) {

      if (listSelection.length == 0) {
	systemStatusTextArea.setText("");
	systemStatusTextArea.append("Please select a Security Level and Press the Start Button. \n");
      }

      if (listSelection.length != 0) {
	for (int k=0; k<listSelection.length; k++) {
	  atomIO.startScl(listSelection[k]);
	}
      }
    } // miStartButton


    // *** Launch the Tool Interface Window ***
    if (source == miToolButton) {
      ToolInterface tl;
      tl = new ToolInterface();
      tl.setSize(700,500);
      tl.setVisible(true);
    } // miToolButton
    

    // *** Launch the SCL Interface Window ***
    if (source == miSclButton) {
      SclInterface sl;
      sl = new SclInterface();
      sl.setSize(700,500);
      sl.setVisible(true);
    } // miSclButton

    
    // *** Stop SCL's ***
    if (source == miStopButton) {
      if (listSelection.length == 0) {
	systemStatusTextArea.setText("");
	systemStatusTextArea.append("Please select a Security Level and Press the Stop Button. \n");
      }
       
      if (listSelection.length != 0) {
	for (int k=0; k<listSelection.length; k++) {
	  atomIO.stopScl(listSelection[k]);
	}
      }
    } // miStopButton       
      
	  
    if (source == quitItem) {
      atomIO.cleanUp();
      System.exit(0);
    } // quitItem

    if (source == setDefaultItem) {
      DefaultWindow settings = new DefaultWindow(this, "Default Settings");
      settings.setVisible(true);
    } // setDefualtItem
 
  } // actionPerformed



  // *********************************************************************************
  // public boolean handleEvent(Event evt)
  //
  // Method Description:  This method handles all of the events that are not of type
  // ACTION_EVENT, generated by the components in the main interface.  The events will
  // include LIST_SELECT, LIST_DESELECT, and WINDOW_DESTROY.
  //
  // Method Specification:  If one of the preceeding events occurs this method will 
  // handle it.  Events are generated by the user activating a component.  Such as, 
  // selecting or deselecting list elements, or destroying the current window/frame.
  // The following lists the events and reactions for each event:
  //
  //        Action Type     Component/Event      Reaction
  //        -----------     ---------------      --------
  //      * LIST_SELECT     sclTypesList         listSelection = selected item
  //                        registeredToolsList  systemTextArea = "non functional"
  //      * LIST_DESELECT   sclTypesList         listSeleciton = selected item
  //      * WINDOW_DESTROY  this                 exit
  //
  // This method will be accessed many times.  Only the WINDOW_DESTROY will be 
  // accessed once.
  //
  // Pre-Condition:  Component objects have been instantiated.
  //
  // Post-Condition: None.
  // 
  // *********************************************************************************

  public void itemStateChanged(ItemEvent evt) {


    //public boolean handleEvent(Event evt) {
    //  if (evt.id == Event.WINDOW_DESTROY) { 
    //    atomIO.cleanUp();
    //    System.exit(0);
    //  }
    
    // *** SCL List ***
    if (evt.getSource().equals(sclTypesList)) {
      listSelection = sclTypesList.getSelectedItems();
    }

    // *** Registered Tools List ***
    if (evt.getSource().equals(registeredToolsList)) {
      systemStatusTextArea.append("The Registered Tools list is for display purposes ");
      systemStatusTextArea.append("and is non-functional.\n");
    }
     
  }  // handleEvent
  


  // *********************************************************************************
  // public void createRegisteredAuditTools ()
  //
  // Method Description:  This method instantiates the registeredToolsList and fills 
  // it with the names of all the currently registered audit tools.
  //
  // Method Specification:  First this method instantiates the registeredToolsList.  
  // This list is a no-selection list, but it defaults to a single selection list.
  // Then the list is filled through the fillList method located in the FileIO
  // class. 
  //   
  // This method will only be accessed once by the constructor.
  //
  // Pre-Condition:  registeredToolsList has been declared.
  //
  // Post-Condition: None.
  // 
  // *********************************************************************************
  
  public void createRegisteredAuditTools () {

    registeredToolsList = atomIO.fillList(registeredToolsList, auditToolsFile, 
					   ToolInterface.NAME);
    registeredTools = registeredToolsList.getItems();
  } // createRegisteredAuditTools



  // *********************************************************************************
  // public void createSclStatusList ()
  //
  // Method Description:  This method instantiates the sclTypesList and fills 
  // it with the names of all the currently registered SCL's.
  //
  // Method Specification:  First this method instantiates the sclTypesList.  This 
  // list is a multiple selection list.  Then the list is filled through the fillList 
  // method located in the FileIO class. 
  //   
  // This method will only be accessed once by the constructor.
  //
  // Pre-Condition:  SclTypesList has been declared.
  //
  // Post-Condition: None.
  // 
  // *********************************************************************************
 
  public void createSclStatusList () {


    sclTypesList = atomIO.fillList(sclTypesList, sclTypesFile, ToolInterface.NAME);

  } // createSclStatusList
  


class WinAdapter extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
    System.exit(0);
  }
}



  // *********************************************************************************
  // public static void main(String[] args) 
  //
  // Method Description:  This method executes Atom.
  //
  // Method Specification: This method initializes the Atom class, resizes the window, 
  // and shows the window.  In other words, it displays the graphical user interface
  // constructed by the Atom Class constructor.
  //
  // This method will only be accessed once.
  //
  // Pre-Condition: None.
  //
  // Post-Condition: None. 
  // 
  // *********************************************************************************

  public static void main(String[] args) {  
    Frame f = new Atom();   
    f.pack();
    f.setVisible(true);
  } // main

} // Atom



















