/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// ***************************************************************************
// ToolInterface Class
// 
//
// Class Description:  
// ------------------
// Purpose:  This class defines the layout and functionality of the Tool Interface.
// This interface is a seperate window from the main interface.  This class instantiates
// and lays out the GUI components for the interface.  It implements its own 
// event handling and is capable of interacting with the whole system.  If this
// window is destroyed, only it will be destroyed.  This interface allows the
// user to add, remove, configure, start, stop, and view audit tools.
//
// System Relationship:  Intercommunication with the whole system is minimal.  The
// majority of system relationships involves updating List or Choice components 
// located on other interfaces.  For example, updating the registeredToolsList in
// the Main Interface or the Atom.class.  File input/output is performed.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.0.
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  Atom.class.
//
// Post Conditions: True.
//
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class must layout the interface components for the Tool Interface onto the 
// screen.  The Tool Interface appears on the console when the "Tool" button on the
// main interface is activated by the user.  This class will provide these options:
// add, remove, configure, start, stop, and view.  These options will be layed out
// in a tabular format for the user to choose from.  This class must respond to user
// generated events.  The following is a list of requirements used by this class:
//
//    *  Define audit tool parsing language (Atpl) keywords
//    *  Instantiate java.awt components used by the GUI
//    *  Layout the java.awt components
//    *  Provide event handling for the java.awt components
//
//
// Constructor
// -----------
//      public ToolInterface()
//
//
// Methods
// -------
//      public void layoutAddPanel()
//      public void layoutRemovePanel()
//      public void layoutConfigurePanel()
//      public void layoutStartPanel()
//      public void layoutStopPanel()
//      public void layoutViewPanel()
//      public void parseParameterText(String text)
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height,
//                      int fill, int anchor, double weight_x, double weight_y,   
//                      int top, int left, int bottom, int right)
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height) 
//      public void constrain(Container container, Component component,
//                      int grid_x, int grid_y, int grid_width, int grid_height,
//                      int top, int left, int bottom, int right)
//
// Event Handling 
// --------------
//      public boolean action(Event event, Object arg)
//      public boolean handleEvent(Event evt)
//
//
// External Classes Used
// ---------------------
//      FileIO
//      Box
//
// *****************************************************************************

import java.awt.*;
import java.lang.*;
import java.util.*;
import java.awt.event.*;

public class ToolInterface extends Constrain implements ActionListener, ItemListener {

  // *******************************************
  // Interface Component Declarations
  // *******************************************
  Panel   tabs;                 // Sub-container for the buttons
  Panel   cards;                // Sub-container for the button features
  Panel   addPanel;             // Sub-container for the Add feature components
  Panel   removePanel;          // Sub-container for the Remove feature components
  Panel   configurePanel;       // Sub-container for the Configure feature components
  Panel   startPanel;           // Sub-container for the Start feature components
  Panel   stopPanel;            // Sub-container for the Stop feature components
  Panel   viewPanel;            // Sub-container for the View feature components

  Button  addButton;            // Add button
  Button  removeButton;         // Remove button
  Button  configureButton;      // Configure button
  Button  startButton;          // Start button
  Button  stopButton;           // Stop button
  Button  viewButton;           // View button
       
  Button  addSubmitButton;        // Add Submit Button
  Button  removeSubmitButton;     // Remove Submit Button
  Button  configureSubmitButton;  // Configure Submit Button
  Button  startSubmitButton;      // Start Submit Button
  Button  stopSubmitButton;       // Stop Submit Button
  Button  continueButton;         // Continue button 

  TextField  addNameTextField;           // Add Feature: Name text field
  TextField  addParameterTextField;      // Add Feature: Parameter text field
  TextField  startParametersTextField;   // Start Feature: Parameter text field
  TextField  startDirectoryTextField;    // Start Feature: Directory text field

  TextArea   addDescriptionTextArea;     // Text area for the Add description
  TextArea   configureTextArea;          // Text area for the Configure current configuration
  TextArea   configureTextAreaEdit;      // Text area for the Configure new configuration
  static TextArea   viewTextArea;        // Text area for the View feature
  
  List  removeToolList;              // Tool List for the Remove feature
  List  configureToolList;           // Tool List for the Configure feature
  List  viewToolList;                // Tool List for the View feature
  List  startToolList;               // Tool List for the Start feature
  List  stopToolList;                // Tool List for the Stop feature
  
  static Choice  addChoice;          // Choice menu for the Add feature
         Choice  configureChoice;    // Choice menu for the Configure feature
  static Choice  startChoice;        // Choice menu for the Start feature
  
  Checkbox startCheckBox;            // Checkbox for the Assign feature
  
  CardLayout singleCard   = new CardLayout();      // Layout Manager for the features / cards
  GridBagLayout  gridbag  = new GridBagLayout();   // Layout Manager for the components
  
  public static String auditToolName;              // Variable used by AddWindow Class

  
  // *******************************************
  // Data Declarations
  // *******************************************

  static  String NAME        = "NAME";              // Atpl keyword 
          String SCLTYPE     = "SCLTYPE";           // Atpl keyword  
          String PARAMETER   = "PARAMETER";         // Atpl keyword  
  static  String DESCRIPTION = "DESCRIPTION";       // Atpl keyword  
  static  String TOOLCLASS   = "TOOLCLASS";         // Atpl keyword  
  static  String DIRECTORY   = "DIRECTORY";         // Atpl keyword  
  static  String DEFAULT     = "DEFAULT";           // Atpl keyword  
  static  String END         = "END";               // Atpl keyword  
  static  String ENDSYMBOL   = "#@";                // Atpl symbol

  static  int    proceed       = 0;                 // Flag set by AddWindow.classs
 
  private int[]  listSelection        = new int[0];      // List selection elements (integer indexes)
  private String[] alphaListSelection = new String[0];   // List selection elements (alpha names)
  private String selectedItem;

  // *******************************************
  // Object instantiation
  // *******************************************
  
  FileIO     toolIO  = new FileIO();



  // *****************************************************************************
  // public ToolInterface()
  //
  // Constructor Description:  The constructor for the ToolInterface  class.  Initializes
  // new variables and components, and lays out the components on the tool interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the tool interface.  It uses the GridBagLayout Manager.  In addition
  // it defines the variables from the Data Declarations.  The following is a listing
  // of the interface layout and its corresponding components:
  // 
  //   Position  Description             Components
  //   --------  -----------             ----------
  //    * Frame  Tool Interface:         GridBagLayout Manager
  //    * North  Tabs/Features Panel     GridLayout Manager, addButton, removeButton, 
  //                                     configureButton, startButton, stopButton, and
  //                                     viewButton
  //    * Center Cards                   CardLayout Manager
  //             Add                     addPanel, addChoice, addNameTextField, 
  //                                     addParameterTextField, addDescriptionTextArea 
  //             Remove                  removePanel, removeToolList, panel1
  //             Configure               configurePanel, configureToolList, configureChoice,
  //                                     configureTextArea, configureTextAreaEdit,panel1,
  //                                     panel2
  //             Start                   startPanel, startParametersTextField, 
  //                                     startDirectoryTextField, startToolList, startChoice,
  //                                     startCheckBox, panel1, panel2
  //             Stop                    stopPanel, stopToolList, panel1
  //             View                    viewPanel, viewToolList, viewTextArea                
  //
  // The constructor will be accessed each time ToolInterface , is instantiated.
  // 
  // Pre-Conditions:  True.
  //
  // Post-Conditions: True.
  //
  // *****************************************************************************

  public ToolInterface () {
    setTitle("Tool Options");

    addWindowListener(new WinAdapter());
    
    // *** Initialize Buttons ****
    addButton        = new Button("Add");
    removeButton     = new Button("Remove");
    configureButton  = new Button("Configure");
    startButton      = new Button("Start");
    stopButton       = new Button("Stop");
    viewButton       = new Button("View");
    continueButton   = new Button("continue");

    // *** BUTTON LISTENERS ***
    addButton.addActionListener(this);
    removeButton.addActionListener(this);
    configureButton.addActionListener(this);
    startButton.addActionListener(this);
    stopButton.addActionListener(this);
    viewButton.addActionListener(this);
    continueButton.addActionListener(this);

    // *** Set Font and Color of Buttons ****
    addButton.setFont(new Font("Dialog", Font.BOLD, 14));
    removeButton.setFont(new Font("Dialog", Font.BOLD, 14));
    configureButton.setFont(new Font("Dialog", Font.BOLD, 14));
    startButton.setFont(new Font("Dialog", Font.BOLD, 14));
    stopButton.setFont(new Font("Dialog", Font.BOLD, 14));
    viewButton.setFont(new Font("Dialog", Font.BOLD, 14));
    continueButton.setFont(new Font("Dialog", Font.BOLD, 12));
    continueButton.setForeground(Color.blue);

    // *** Tabs ****
    tabs = new Panel();
    tabs.setLayout(new GridLayout(0,6));
     tabs.add(addButton);
     tabs.add(removeButton);   
     tabs.add(configureButton);
     tabs.add(startButton);
     tabs.add(stopButton);
     tabs.add(viewButton);
    add("North", tabs);

    // *** Layout Card Method ****
    layoutAddPanel();

    // *** Card ****
    cards = new Panel();
    cards.setLayout(singleCard);
    cards.add("Add",       addPanel);
    add("Center", cards);
  }  // Constructor



  // ***************************************************************************** 
  // public boolean action(Event evt, Object arg)
  //
  // Method Description:  This method handles all of the events (ACTION_EVENT) 
  // generated by the components in the tool interface.
  //
  // Method Specification:  If an ACTION_EVENT occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ACTION_EVENT:
  //
  //        Label     Component/Event       Reaction
  //        -----     ---------------       --------
  //      * Add       continueButton        First interface of adding an audit tool, when
  //                                        this button is clicked the second interface
  //                                        appears as a dialog box     
  //      * Remove    removeSubmitButton    Remove an audit tool from the registered audit
  //                                        tools
  //      * Configure configureSubmitButton Modify the user selected field of the audtit
  //                                        tool
  //      * Start     startSubmitButton     Start the user selected audit tool
  //      * Stop      stopSubmitButton      Stop the user selected audit tool
  //
  // This method will be accessed many times.
  //
  // Pre-Condition:  Component objects have been instantiated.
  //
  // Post-Condition: True. Events handle properly.
  //
  // *****************************************************************************

  public void actionPerformed(ActionEvent evt) {
    Object source = evt.getSource();
    
    String nameInfo        = "";
    String sclInfo         = "";
    String parmInfo        = "";
    String directoryInfo   = "";
    String descriptionInfo = "";
    String fieldData       = "";
    int    tempIndex;

    // ----------------------
    // Add Continue Button
    // ----------------------
    if (source == continueButton) {
      
      // Collect information from the TextFields.
      // Collection Format: <name> <scl type> <parameters> <description>
      nameInfo = addNameTextField.getText();
      sclInfo  = addChoice.getSelectedItem();
      parmInfo = addParameterTextField.getText();
      descriptionInfo = addDescriptionTextArea.getText();        
      nameInfo = nameInfo.trim();                                // remove whitespace from ends

      // Validity check
      if (nameInfo.equals("")) {
	Atom.systemStatusTextArea.setText("The name field is required to Add a tool. \n");
	return;
      }
      
      // Variable used by AddWindow Class
      auditToolName = nameInfo;
      
      // Initialize and show the AddWindow Dialog.
      AddWindow popup = new AddWindow(this, "Add Tool");
      popup.setVisible(true);

      ///
      // If the user clicked the Submit button in the AddWindow Dialog.
      // Then do the following:
      //        * Put the Audit Tool Data into the audit_tools.dat file
      //        * Data is in the format <name> <scl type> <tool class> 
      //          <directory> <default> <description> <end symbol>
      //        * Update the registered audit tools list components
      //        * Clear the TextFields of the Add interface
      //
      //        * Update internal list array
      if (proceed == 1) {	  
	toolIO.AtplAppendText(Atom.auditToolsFile, NAME, nameInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, SCLTYPE, sclInfo);
	parseParameterText(parmInfo);
	  
	toolIO.AtplAppendText(Atom.auditToolsFile, TOOLCLASS, AddWindow.toolClassInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, DIRECTORY, AddWindow.directoryInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, DEFAULT,   AddWindow.defaultInfo);     
	
	toolIO.AtplAppendText(Atom.auditToolsFile, DESCRIPTION, descriptionInfo);
	toolIO.AtplAppendText(Atom.auditToolsFile, END, ENDSYMBOL);
	
	// Update registered audit tools
	Atom.registeredToolsList.add(nameInfo);

	// The Scl class may not have been instantiated yet, so 
	// check and handle these lists if possible.
	try { 	       
	  SclInterface.assignToolList.add(nameInfo);
	  SclInterface.unassignToolList.add(nameInfo); 	 
	}
	catch (NullPointerException e) {
	  String err = e.toString();
	}

	// Clear the text fields
	addNameTextField.setText("");
	addParameterTextField.setText("");
	addDescriptionTextArea.setText("");	

	// ----- 4/7/98 ------------------------------------------
	// udpate internal list array
	Atom.registeredTools = addToList(Atom.registeredTools, nameInfo);
	//for (int z=0; z<Atom.registeredTools.length; z++) {
	//System.out.println(Atom.registeredTools[z]);
	//}

      }
    } // add continueButton


    // ----------------------
    // Remove Button
    // ----------------------
    if (source == removeSubmitButton) {
      
      // Validity check
      if (selectedItem == null) {
	Atom.systemStatusTextArea.setText("Please select an Item to Remove. \n");
	return;
      }
      else {
	//
	// The user has selected an item to remove so do the following:
	//     * Remove the item from the audit_tool.dat file
	//     * Remove the item from all audit tools lists
	//
	// ------- 4/6/98 ------------------------------------
	//Atom.registeredTools = removeFromList(Atom.registeredTools, item);

	//remove from all audit tool lists and update
	removeToolList.remove(selectedItem);
	Atom.registeredToolsList.remove(selectedItem);

	// The Scl class may not have been instantiated yet, so 
	// check and handle these lists if possible.
	try { 	       
	  SclInterface.assignToolList.remove(selectedItem); 	       
	  SclInterface.unassignToolList.remove(selectedItem); 	       
	}
	catch (NullPointerException e) {
	  String err = e.toString();
	}
	
	// Update audit_tools.dat file
	toolIO.AtplRemoveRecord(Atom.auditToolsFile, Atom.auditTempFile, selectedItem);	   
      }
    } // remove button


    // ----------------------
    // Configure Button
    // ----------------------
    if (source == configureSubmitButton) {
      String choiceItem = "";
      String info = "";
      
      // Collect information from the Choice menu and the TextArea
      choiceItem = configureChoice.getSelectedItem();
      info = configureTextAreaEdit.getText();
      info = info.trim();
      
      // Validity check
      if (!(alphaListSelection.length > 0)) {
	Atom.systemStatusTextArea.append("Please select an item from the list to configure.\n");
	return;
      }

      // Validity check
      if (info.equals("")) {
	Atom.systemStatusTextArea.append("Please enter New Configuration information. \n");
	return;
      }

      //
      // Based on the user selected field from the Choice menu:
      //     * Replace the feild in the record with the information received in the TextArea
      //     * Clear the TextArea's
      //     * Display new information in TextArea
      //
      toolIO.AtplReplaceRecord(Atom.auditToolsFile, Atom.auditTempFile, alphaListSelection[0], 
			       choiceItem, info);
      configureTextArea.setText("");
      configureTextAreaEdit.setText("");
      toolIO.AtplGetRecord(Atom.auditToolsFile, alphaListSelection[0], configureTextArea);  
    } // Configure Button


    // ----------------------
    // Start Button
    // ----------------------
    if (source == startSubmitButton) {

      // Validity check
      if (!(alphaListSelection.length > 0)) {
	Atom.systemStatusTextArea.append("Please select an item from the list to Start.\n");
	return;
      }
	 
      // Collect information from the TextFields
      parmInfo = startParametersTextField.getText();
      directoryInfo = startDirectoryTextField.getText();

      if (!(directoryInfo.endsWith("/"))) {            // if directory does not end with "/" then put it there
	directoryInfo = directoryInfo+"/";
      }	 

      if (parmInfo.equals("")) {
	Atom.systemStatusTextArea.append("Please enter the Execution Command for the audit tool. \n");
	return;
      }
	 
      if (directoryInfo.equals("")) {
	Atom.systemStatusTextArea.append("Please enter a Directory path for the audit tool. \n");
	return;
      }
	
      // If user checks the CheckBox then assign the audit tool to the corresponding Scl Level
      if (startCheckBox.getState()) {
	String tempInfo = startChoice.getSelectedItem();
	toolIO.appendSclAssignment(tempInfo, tempInfo, parmInfo, directoryInfo);  
      }
	 
      // Start the audit tool
      toolIO.startTool(directoryInfo, alphaListSelection[0], parmInfo);
    } // Start


    // ----------------------
    // Stop Button
    // ----------------------
    if (source == stopSubmitButton) {

      // Validity check
      if (!(alphaListSelection.length >0)){
	Atom.systemStatusTextArea.append("Please select an item from the list to Stop.\n");
	return;
      }

      // Stop audit tool
      toolIO.stopTool(alphaListSelection[0]);	 
    }
       

    // ----------------------
    // Cards
    // ----------------------
    if (source instanceof Component &&
	((Component)evt.getSource()).getParent().equals(tabs)) {

      String tabType = evt.getActionCommand();

 
      //  
      // If the user selects a tab/feature display the appropriate card, moreover
      // display the card/features interface
      //
      if (tabType.equals("Add")) {
	layoutAddPanel();
	cards.add("Add", addPanel); 
      }      

      if (tabType.equals("Remove")) {
	layoutRemovePanel();
	selectedItem=null;
	cards.add("Remove", removePanel); 
      }	

      if (tabType.equals("Configure")) {
	layoutConfigurePanel();
	cards.add("Configure", configurePanel); 
	alphaListSelection = new String[0];
      }	

      if (tabType.equals("Start")) {
	layoutStartPanel();
	cards.add("Start", startPanel); 
	alphaListSelection = new String[0];
      }	
	
      if (tabType.equals("Stop")) {
	layoutStopPanel();
	cards.add("Stop", stopPanel); 
	alphaListSelection = new String[0];
      }	

      if (tabType.equals("View")) {
	layoutViewPanel();
	cards.add("View", viewPanel); 
	alphaListSelection = new String[0];
      }	

      singleCard.show(cards, tabType);   
    }

  } // action


class WinAdapter extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
    dispose();
  }
}


  // *********************************************************************************
  // public boolean handleEvent(Event event)
  //
  // Method Description:  This method handles all of the events that are not of type
  // ACTION_EVENT, generated by the components in the tool interface.  The events will
  // include LIST_SELECT, LIST_DESELECT, and WINDOW_DESTROY.
  //
  // Method Specification:  If one of the preceeding events occurs this method will 
  // handle it.  Events are generated by the user activating a component.  Such as, 
  // selecting or deselecting list elements, or destroying the current window/frame.
  // The following lists the events and reactions for each event:
  //
  //        Action Type     Component/Event        Reaction
  //        -----------     ---------------        --------
  //      * LIST_SELECT     removeToolList         listSelection = selected indexes
  //                        configureToolsList     alphaListSelection = selected items
  //                        startToolList          alphaListSelection = selected items
  //                        stopToolList           alphaListSelection = selected items
  //                        viewToolList           alphaListSelection = selected items
  //      * LIST_DESELECT   removeToolLst          listSelection = selected indexes
  //                        configureToolsList     alphaListSelection = selected items
  //                        startToolList          alphaListSelection = selected items
  //                        stopToolList           alphaListSelection = selected items
  //                        viewToolList           alphaListSelection = selected items
  //      * WINDOW_DESTROY  this                 exit
  //
  // This method will be accessed many times.  Only the WINDOW_DESTROY will be 
  // accessed once.
  //
  // Pre-Condition:  Component objects have been instantiated.
  //
  // Post-Condition: True.  Lists have proper component names or indexes.
  // 
  // *********************************************************************************

  public void itemStateChanged(ItemEvent event) {

    String fieldData = "";

    //    if (event.id == Event.WINDOW_DESTROY) {
    //      this.dispose();
    //      return super.handleEvent(event);
    //  }
    
    switch(event.getStateChange()) {
    case ItemEvent.SELECTED:

      // Remove
      if (event.getSource().equals(removeToolList)) {
	selectedItem = removeToolList.getSelectedItem();
      }    

      // Configure
      //     * Get List selection
      //     * Clear TextField
      //     * Place record information in TextField based on List selection
      //
      if (event.getSource().equals(configureToolList)) {
      	alphaListSelection = configureToolList.getSelectedItems();
	configureTextArea.setText("");
	toolIO.AtplGetRecord(Atom.auditToolsFile, alphaListSelection[0], configureTextArea);      
      }

      // Start
      //     * Get List selection
      //     * Clear TextFields
      //     * Get field information from audit tool records based on List selection
      //     * Place information in appropriate TextField
      //
      if (event.getSource().equals(startToolList)) {
	alphaListSelection = startToolList.getSelectedItems();
	startParametersTextField.setText("");
	startDirectoryTextField.setText("");
	fieldData = toolIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], DEFAULT);
	startParametersTextField.setText(fieldData);
	fieldData = toolIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], DIRECTORY);
	startDirectoryTextField.setText(fieldData);
      }

      // Stop
      if (event.getSource().equals(stopToolList)) {
	alphaListSelection = stopToolList.getSelectedItems();	
      }

      // View
      //    * Get List selection
      //    * Clear TextField
      //    * Place information in TextField
      //
      if (event.getSource().equals(viewToolList)) {
	alphaListSelection = viewToolList.getSelectedItems();
	viewTextArea.setText("");
	toolIO.AtplGetRecord(Atom.auditToolsFile, alphaListSelection[0], viewTextArea);
      }
      
    break;

    case ItemEvent.DESELECTED:
      
      // Configure
      if (event.getSource().equals(configureToolList)) {
	alphaListSelection = configureToolList.getSelectedItems();		
      }

      // Start
      if (event.getSource().equals(startToolList)) {
	alphaListSelection = startToolList.getSelectedItems();	
      }
      
      // Stop
      if (event.getSource().equals(stopToolList)) {
	alphaListSelection = startToolList.getSelectedItems();	
      }            

      // View
      if (event.getSource().equals(viewToolList)) {
	alphaListSelection = viewToolList.getSelectedItems();	
      }
    break;
    } // switch 
    
  }



  // *********************************************************************************
  // public void layoutAddPanel ()
  //
  // Method Description:  This method lays out all of the components for the Add tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Add tab is activated by the user, the corresponding
  // Add feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Add tab is activated this
  // method will be called.
  //
  // Pre-Condition:  Add components have been declared.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutAddPanel () {
    addPanel                 = new Panel();
    addChoice                = new Choice();
    addNameTextField         = new TextField(30);
    addParameterTextField    = new TextField(40);
    addDescriptionTextArea   = new TextArea(6,60);

     addPanel = new Panel();    
     addPanel.setLayout(gridbag);
     
     addChoice = toolIO.fillChoice(addChoice, Atom.sclTypesFile, NAME);

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.
     constrain(addPanel, new Label("Name:"), 0,0,1,1, 0,20,0,0);
     constrain(addPanel, addNameTextField, 0,1,1,1, 0,20,0,0);     
     constrain(addPanel, new Label("SCL Type:"), 0,2,1,1, 10,20,0,0);
     constrain(addPanel, addChoice, 0,3,1,1, 0,20,0,0);    
     constrain(addPanel, new Label("Parameters:"), 0,4,1,1, 10,20,0,0);
     constrain(addPanel, addParameterTextField, 0,5,1,1, 0,20,0,0);     
     constrain(addPanel, new Label("Description:"), 0,6,1,1, 10,20,0,0);
     constrain(addPanel, addDescriptionTextArea, 0,7,1,1, 0,20,0,0);   
     constrain(addPanel, continueButton, 0,8,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

  }  // layoutAddPanel


  // *********************************************************************************
  // public void layoutRemovePanel ()
  //
  // Method Description:  This method lays out all of the components for the Remove tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Remove tab is activated by the user, the corresponding
  // Remove feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Remove tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Remove components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutRemovePanel () {
    removePanel         = new Panel();             
    removeToolList      = new List(15, false);       // List for audit tools, SS
    Panel panel1        = new Panel();
   
     panel1.setLayout(gridbag);
     removePanel.setLayout(gridbag);
     removeSubmitButton = new Button("Submit");
     removeSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     removeSubmitButton.setForeground(Color.blue);

     // ADD LISTENERS
     removeSubmitButton.addActionListener(this);
     removeToolList.addItemListener(this);

     // Fill the removeToolList with the names of the registered audit tools
     for (int z=0; z<Atom.registeredTools.length; z++) {
       removeToolList.add(Atom.registeredTools[z]);
     }

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.          
     constrain(panel1, removeToolList, 0,0,1,1);
     Box panelBox = new Box(panel1, "Audit Tools");
     panelBox.etchedOut();
     
     constrain(removePanel, panelBox, 0,0,1,1);
     constrain(removePanel, removeSubmitButton, 0,1,1,1, 20,40,0,0);

  } // layoutRemovePanel()


  // *********************************************************************************
  // public void layoutConfigurePanel ()
  //
  // Method Description:  This method lays out all of the components for the Configure tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Configure tab is activated by the user, the corresponding
  // Configure feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Configure tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Configure components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutConfigurePanel() {
    configurePanel              = new Panel();
    configureToolList           = new List(22, false);
    configureChoice             = new Choice();
    configureTextArea           = new TextArea(8,50);
    configureTextAreaEdit       = new TextArea(8,50);

    Panel     panel1            = new Panel();
    Panel     panel2            = new Panel();
    
     configurePanel.setLayout(gridbag);
     panel1.setLayout(gridbag);
     panel2.setLayout(gridbag);
     configureSubmitButton = new Button("Submit");
     configureSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     configureSubmitButton.setForeground(Color.blue);

     // ADD LISTENERS
     configureSubmitButton.addActionListener(this);
     configureToolList.addItemListener(this);

     //--- 4/7/98 ----------------
     // Fill the configureToolList with the names of the registered audit tools     
     for (int z=0; z<Atom.registeredTools.length; z++) {
       configureToolList.add(Atom.registeredTools[z]);
     }
     
     // This is where the choice menu is filled with choices
     configureChoice.add(NAME);
     configureChoice.add(SCLTYPE);
     configureChoice.add(PARAMETER);
     configureChoice.add(TOOLCLASS);
     configureChoice.add(DIRECTORY);
     configureChoice.add(DEFAULT);
     configureChoice.add(DESCRIPTION);
     
      constrain(panel1, configureToolList, 0,0,1,1);
      Box panelBox = new Box(panel1, "Audit Tools");
      panelBox.etchedOut();

      // The constrain method places the components onto the interface using the
      // GridBagLayout manager.               
      constrain(panel2, configureChoice, 0,0,1,1, 10,0,0,0);
      constrain(panel2, new Label("Current Configuration"), 0,1,1,1, 20,0,0,0);
      constrain(panel2, configureTextArea, 0,2,1,1);
      constrain(panel2, new Label("New Configuration"), 0,3,1,1, 10,0,0,0);
      constrain(panel2, configureTextAreaEdit, 0,4,1,1); 
      constrain(panel2, configureSubmitButton, 0,5,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

      constrain(configurePanel, panelBox, 0,0,1,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);
      constrain(configurePanel, panel2, 1,0,1,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);    	

  } // layoutRemovePanel()


  // *********************************************************************************
  // public void layoutStartPanel ()
  //
  // Method Description:  This method lays out all of the components for the Start tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Start tab is activated by the user, the corresponding
  // Start feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Start tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Start components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutStartPanel() {
    startPanel                      = new Panel();
    startParametersTextField        = new TextField(40);
    startDirectoryTextField         = new TextField(40);
    startToolList                   = new List(15, false);
    startChoice                     = new Choice();
    startCheckBox                   = new Checkbox(" Assign Tool To SCL");

    Panel      panel1               = new Panel();
    Panel      panel2               = new Panel();
   
     startPanel.setLayout(gridbag);
     panel1.setLayout(gridbag);
     panel2.setLayout(gridbag);
     startSubmitButton = new Button("Submit");
     startSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     startSubmitButton.setForeground(Color.blue);

     // ADD LISTENERS
     startSubmitButton.addActionListener(this);
     startToolList.addItemListener(this);

     // Fill the startToolList with the names of the registered audit tools
     // Fill the choice menu with the names of the SCL types
     startToolList = toolIO.fillList(startToolList, Atom.auditToolsFile, NAME);
     startChoice   = toolIO.fillChoice(startChoice, Atom.sclTypesFile, NAME);

     constrain(panel1, startToolList, 0,0,1,1);
     Box panelBox = new Box(panel1, "Audit Tools");
     panelBox.etchedOut();

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.                    
     constrain(panel2, startCheckBox, 0,0,1,1, 10,0,0,0);
     constrain(panel2, new Label("SCL Type:"), 0,1,1,1, 10,0,0,0);
     constrain(panel2, startChoice, 0,2,1,1);    
     constrain(panel2, new Label("Execution Command:"), 0,3,1,1, 20,0,0,0);
     constrain(panel2, startParametersTextField, 0,4,1,1);
     constrain(panel2, new Label("Directory:"), 0,5,1,1, 10,0,0,0);
     constrain(panel2, startDirectoryTextField, 0,6,1,1);
     constrain(panel2, startSubmitButton, 0,7,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.CENTER, 1.0, 1.0, 40, 0, 5, 5);
     
     constrain(startPanel, panelBox, 0,0,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);
     constrain(startPanel, panel2, 1,0,1,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);

  } // layoutStartPanel()


  // *********************************************************************************
  // public void layoutStopPanel ()
  //
  // Method Description:  This method lays out all of the components for the Stop tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Stop tab is activated by the user, the corresponding
  // Stop feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Stop tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Stop components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutStopPanel() {
    stopPanel           = new Panel();               
    stopToolList        = new List(15, false);       // List for audit tools, SS
    
    Panel panel1        = new Panel();  

     panel1.setLayout(gridbag);
     stopPanel.setLayout(gridbag);
     stopSubmitButton = new Button("Submit");
     stopSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     stopSubmitButton.setForeground(Color.blue);

     // ADD LISTENERS
     stopSubmitButton.addActionListener(this);
     stopToolList.addItemListener(this);

     // Fill the stopToolList with the names of the registered audit tools
     stopToolList = toolIO.fillList(stopToolList, Atom.auditToolsFile, NAME);

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.               
     constrain(panel1, stopToolList, 0,0,1,1);
     Box panelBox = new Box(panel1, "Audit Tools");
     panelBox.etchedOut();
    
     constrain(stopPanel, panelBox, 0,0,1,1);
     constrain(stopPanel, stopSubmitButton, 0,1,1,1, 20,40,0,0);

  } // layoutStopPanel()


  // *********************************************************************************
  // public void layoutViewPanel ()
  //
  // Method Description:  This method lays out all of the components for the View tab,
  // located on the tool interface.
  //
  // Method Specification:  When the View tab is activated by the user, the corresponding
  // View feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the View tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  View components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutViewPanel() {
    viewPanel                    = new Panel();
    viewToolList                 = new List(15, false);
    viewTextArea                 = new TextArea(20,50);

    Panel     panel1             = new Panel();
    Panel     panel2             = new Panel();

     viewPanel.setLayout(gridbag);
     panel1.setLayout(gridbag);
     panel2.setLayout(gridbag);

     // ADD LISTENERS
     viewToolList.addItemListener(this);

     // Fill the viewToolList with the names of the registered audit tools   
     viewToolList = toolIO.fillList(viewToolList, Atom.auditToolsFile, NAME);     

     // The constrain method places the components onto the interface using the
     // GridBagLayout manager.                        
     constrain(panel1, viewToolList, 0,0,1,1);
     Box panelBox = new Box(panel1, "Audit Tools");
     panelBox.etchedOut();
    
     constrain(panel2, new Label("Audit Tool Specifications"), 0,0,1,1);
     constrain(panel2, viewTextArea, 0,1,1,1);    

     constrain(viewPanel, panelBox, 0,0,1,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 0, 5, 10);
     constrain(viewPanel, panel2, 1,0,1,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.NORTH, 1.0, 1.0, 1, 10, 5, 10);

  } // layoutViewPanel()
    

  // *********************************************************************************
  // public void parseParameterText (String text)
  //
  // Method Description:  This method parses the parameter string passed to it into 
  // seperate tokens/parameters.  
  //
  // Method Specification:  This method parses the parameter string passed to it.  The
  // parsing is delimited by a space (" ").  Each token is considered a parameter for 
  // the audit tool, and it is placed into the audit_tools.dat file in the format:
  // PARAMETER  token.
  //
  // This method will be accessed several times.  Every time the Add tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  True.  The string passed to this method does not have to be in
  // any special format.
  //
  // Post-Condition: True.  All of the individual (parsed) parameters are placed into
  // the audit_tools.dat file.
  // 
  // *********************************************************************************

  public void parseParameterText(String text) {
    String token;
    StringTokenizer inputString = new StringTokenizer(text, " ");
    
    while (inputString.hasMoreTokens()) {
      token = inputString.nextToken();
      toolIO.AtplAppendText(Atom.auditToolsFile, PARAMETER, token);
    } // while

  } // parseParameterText



  // ----------------------------------------------------------------------
  public String[] addToList(String[] list, String item) {
    
    int size = list.length;
    String[] tempList = new String[size+1]; 
    System.arraycopy(list, 0, tempList, 0, list.length);
    tempList[size] = item;
    
    return tempList;
  }

  // ----------------------------------------------------------------------
  public String[] removeFromList(String[] list, String item) {
    
    String[] tempList = new String[list.length-1];
    int tempindex = 0;

    for (int k=0; k<list.length; k++) {
      if (!(list[k].equals(item))) {
	tempList[tempindex] = list[k];
	tempindex++;
      }
    }
    return tempList;
  }
}

 











 




